@@ -9,15 +9,13 @@ use vars qw(@ISA @EXPORT_OK);
 
 @ISA = qw(Exporter);
 
-use UNIVERSAL 'isa';
-
 BEGIN {
     eval 'require Scalar::Util';
     unless ($@) {
 	import Scalar::Util qw(blessed refaddr);
     } else {
 	# Use the pure Perl emulations (directly snagged from Scalar::Util).
-	sub UNIVERSAL::a_sub_not_likely_to_be_here { ref($_[0]) }
+	eval 'sub UNIVERSAL::a_sub_not_likely_to_be_here { ref($_[0]) }';
 	*blessed = sub ($) {
 	    local($@, $SIG{__DIE__}, $SIG{__WARN__});
 	    length(ref($_[0]))
@@ -63,7 +61,10 @@ use overload
     '>'		=> \&is_proper_superset,
     '<='	=> \&is_subset,
     '>='	=> \&is_superset,
-    'bool'	=> \&size;
+    'bool'	=> \&size,
+    '@{}'	=> sub { [ $_[0]->members ] },
+    '='         => sub { $_[0]->new($_[0]->members) },
+    'cmp'       => sub { "$_[0]" cmp "$_[1]" };
 
 use constant OVERLOAD_BINARY_2ND_ARG  => 1;
 use constant OVERLOAD_BINARY_REVERSED => 2;
@@ -96,7 +97,7 @@ sub new {
 
     my $self = { };
 
-    bless $self, $class;
+    bless $self, ref $class || $class;
 
     $self->_new_hook( \@_ );
 
@@ -187,13 +188,19 @@ sub each {
     return $e;
 }
 
-sub _clone {
+sub _empty_clone {
     my $self     = shift;
     my $original = shift;
 
     $self->{'universe'} = $original->{'universe'};
-
     $self->{'null'    } = $original->{'null'    };
+}
+
+sub _clone {
+    my $self     = shift;
+    my $original = shift;
+
+    $self->_empty_clone($original);
 
     $self->_insert( $original->{'elements'} );
 }
@@ -209,6 +216,15 @@ sub clone {
 
 *copy = \&clone;
 
+sub empty_clone {
+    my $self  = shift;
+    my $clone = (ref $self)->new;
+
+    $clone->_empty_clone( $self );
+
+    return $clone;
+}
+
 sub clear {
     my $self = shift;
 
@@ -464,6 +480,91 @@ sub unique {
     return $this->_unique( @_ );
 }
 
+sub _make_cartesian_product_iterator {
+    my @iter;
+    my @value;
+    for my $set (@_) {
+	return unless $set->isa('Set::Scalar');
+	my @member = $set->members;
+	my %member;
+	@member{@member} = @member;
+	push @iter, \%member;
+	push @value, scalar CORE::each(%{ $iter[-1] });
+    }
+    return sub {
+	return unless @iter;
+	my @now = @value;
+	my $ix;
+	for ($ix = $#iter; $ix >= 0; $ix--) {
+	    my $next = CORE::each(%{ $iter[$ix] });
+	    if (defined $next) {
+		$value[$ix] = $next;
+		last;
+	    } else {
+		keys %{ $iter[$ix] };  # Reset the iterator.
+		$value[$ix] = CORE::each(%{ $iter[$ix] });
+	    }
+	}
+	if ($ix < 0) {
+	    @iter = ();  # All done.
+	}
+	return @now;
+    };
+}
+
+sub cartesian_product_iterator {
+    shift unless ref $_[0];
+    return &_make_cartesian_product_iterator;
+}
+
+sub cartesian_product {
+    my $iterator = &cartesian_product_iterator;
+    return unless defined $iterator;
+    my $product = $_[0]->empty_clone;
+    while (my @member = $iterator->()) {
+	$product->insert(\@member);
+    }
+    return $product;
+}
+
+sub _make_power_set_iterator {
+    return unless $_[0]->isa('Set::Scalar');
+    my @member = $_[0]->members; 
+    my @iter   = (0) x @member;
+    return sub {
+	return unless @iter;
+	my $ix;
+	for ($ix = 0; $ix < @iter; $ix++) {
+	    if ($iter[$ix]++ == 0) {
+		last;
+	    } else {
+		$iter[$ix] = 0;
+	    }
+	}
+	if ($ix == @iter) {
+	    @iter = ();  # All done.
+	}
+	return map { $member[$_] } grep { $iter[$_] } 0..$#iter;
+    };
+}
+
+sub power_set_iterator {
+    shift unless ref $_[0];
+    return &_make_power_set_iterator;
+}
+
+sub power_set {
+    my $iterator = &power_set_iterator;
+    return unless defined $iterator;
+    my $power = $_[0]->empty_clone;
+    my @member;
+    do {
+	@member = $iterator->();
+	$power->insert($_[0]->empty_clone->insert(@member));
+    } while (@member);
+    return $power;
+}
+
 sub is_universal {
     my $self = shift;
 
